@@ -292,7 +292,6 @@ int callPrg(int command, char *name, int mode, int pass, int choice, char **retB
    XEvent xevent;
 #endif
    int    answered;
-/*#if DEBUG*/
    if ( debugD4 )
    {
       char *s = "-";
@@ -327,7 +326,7 @@ int callPrg(int command, char *name, int mode, int pass, int choice, char **retB
               function ? function:"-"
               );
    }
-/*#endif*/
+
    /* this portion of code will all time be processed */ 
    if ( name )
    {
@@ -597,7 +596,7 @@ int callPrg(int command, char *name, int mode, int pass, int choice, char **retB
                       break;
                       case GET_IQ:
                           shmem->retVal = getIq(name,  mode&PALL);
-                      break;
+                     break;
                       case CHECK_NOZZLE:
                          shmem->retVal = checkNozzle(name,  mode&PALL);
                       break;
@@ -626,6 +625,28 @@ int callPrg(int command, char *name, int mode, int pass, int choice, char **retB
                          shmem->retVal = 0;
                   }
                   shmem->ready = 1;
+		  if ( errno == ENODEV && deviceHdl > -1 )
+		  {
+		     close(deviceHdl);
+		     errno         = 0;
+		     deviceHdl     = -1;
+		     d4Initialized =  False;
+		     sockId        = -1;
+		     sockIdData    = -1;
+		  }
+		  
+		  if ( errno == -1 && deviceHdl > -1 )
+		  {
+		     /* we had problems with the printer, restart from the begin */
+                     if ( d4Initialized )
+		        exitD4(deviceHdl);
+		     close(deviceHdl);
+		     deviceHdl     = -1;
+		     sockId        = -1;
+		     sockIdData    = -1;
+		     d4Initialized = False;
+		     errno         = 0;
+		  }
                }
                else
                {
@@ -656,7 +677,7 @@ int callPrg(int command, char *name, int mode, int pass, int choice, char **retB
             signal(SIGCHLD, sigChild);
       }
    }
-   
+
    /* set values to our shared memory */
    if ( shmem && !shmem->block )
    {
@@ -676,6 +697,12 @@ int callPrg(int command, char *name, int mode, int pass, int choice, char **retB
    }
    else
    {
+      if ( errno == ENODEV )
+      {
+          if ( deviceHdl > 0 )
+             close(deviceHdl);
+	  deviceHdl = -1;
+      }
       return -1;
    }
 
@@ -797,43 +824,42 @@ int doCommand(int working)
       function = shmem->function;
       name     = shmem->name;
       shmem->buf[0] = '\0'; /* clean return buffer */
-/*#if DEBUG*/
+    
+      if ( debugD4 )
       {
-         if ( debugD4 )
-         {
-           char *s = "-";
-           switch(shmem->command)
-           {
-              case TEST_DEV:       s = "TEST_DEV";       break;
-              case GET_ID:         s = "GET_ID";         break;
-              case GET_STAT:       s = "GET_STAT";       break;
-              case GET_IQ:         s = "GET_IQ";         break;
-              case RESET_PRT:      s = "RESET_PRT";      break;
-              case CHECK_NOZZLE:   s = "CHECK_NOZZLE";   break;
-              case CLEAN_NOZZLE:   s = "CLEAN_NOZZLE";   break;
-              case CLEAN_NOZZLE_C: s = "CLEAN_NOZZLE_C"; break;
-              case CLEAN_NOZZLE_B: s = "CLEAN_NOZZLE_B"; break;
-              case WAIT_STAT_04:   s = "WAIT_STAT_04";   break;
-              case WAIT_STAT_01:   s = "WAIT_STAT_01";   break;
-              case EXCHANGE_ALL:   s = "EXCHANGE_ALL";   break;
-              case EXCHANGE_B:     s = "EXCHANGE_B";     break;
-              case EXCHANGE_C:     s = "EXCHANGE_C";     break;
-              case EXCHANGE_NEXT:  s = "EXCHANGE_NEXT";  break;
-              case ALIGN_HEAD:     s = "ALIGN_HEAD";     break;
-              case ALIGN_HEAD_C:   s = "ALIGN_HEAD_C";   break;
-              case TERMINATE:      s = "TERMINATE";      break;
-              case RELEASE:        s = "RELEASE";        break;
-           }
-           fprintf(stderr,"doCommand(%s, %s,%d,%d,%d,...,%s)\n",
-                   s,
-                   name ? name : "-",
-                   mode,
-                   pass,
-                   choice,
-                   function ? function:"-");
-         }
+        char *s = "-";
+        switch(shmem->command)
+        {
+           case TEST_DEV:	s = "TEST_DEV";       break;
+           case GET_ID: 	s = "GET_ID";	      break;
+           case GET_STAT:	s = "GET_STAT";       break;
+           case GET_IQ: 	s = "GET_IQ";	      break;
+           case RESET_PRT:	s = "RESET_PRT";      break;
+           case CHECK_NOZZLE:	s = "CHECK_NOZZLE";   break;
+           case CLEAN_NOZZLE:	s = "CLEAN_NOZZLE";   break;
+           case CLEAN_NOZZLE_C: s = "CLEAN_NOZZLE_C"; break;
+           case CLEAN_NOZZLE_B: s = "CLEAN_NOZZLE_B"; break;
+           case WAIT_STAT_04:	s = "WAIT_STAT_04";   break;
+           case WAIT_STAT_01:	s = "WAIT_STAT_01";   break;
+           case EXCHANGE_ALL:	s = "EXCHANGE_ALL";   break;
+           case EXCHANGE_B:	s = "EXCHANGE_B";     break;
+           case EXCHANGE_C:	s = "EXCHANGE_C";     break;
+           case EXCHANGE_NEXT:  s = "EXCHANGE_NEXT";  break;
+           case ALIGN_HEAD:	s = "ALIGN_HEAD";     break;
+           case ALIGN_HEAD_C:	s = "ALIGN_HEAD_C";   break;
+           case TERMINATE:	s = "TERMINATE";      break;
+           case RELEASE:	s = "RELEASE";        break;
+        }
+        fprintf(stderr,"doCommand(%s, %s,%d,%d,%d,...,%s)\n",
+        	s,
+        	name ? name : "-",
+        	mode,
+        	pass,
+        	choice,
+        	function ? function:"-");
       }
-/*#endif*/
+      
+
       if ( working == -1 && shmem->command != TERMINATE )
       {
          if ( deviceHdl > -1 )
@@ -902,7 +928,7 @@ int doCommand(int working)
                switch(currentAction)
                {
                   case EXCHANGE_ALL:
-                     if ( deviceHdl )
+                     if ( deviceHdl > 0 )
                         doExchange(name,deviceHdl, PROT_D4, 4, 0);
                }
                if ( working == -1 )
@@ -1234,7 +1260,7 @@ static int openDevice(char *dev, int doTest )
          SET_TIMER(ti,oti,500);   
          i = read(deviceHdl, iBuf, 511 );
 #if DEBUG
-         fprintf(stderr,"Read old mesages from %s -> \n",dev, i);
+         fprintf(stderr,"Read old mesages from %s -> %d\n",dev, i);
 #endif
          RESET_TIMER(ti,oti);
       }
@@ -1299,10 +1325,10 @@ static int exitD4(int deviceHdl)
    /* If the printer expect the D4 Mode, we will */
    /* get an error message (malformed packet)    */
    /* else the printer ID                        */
-#if 0
+#if DEBUG
    {
    int i;
-   fprintf(stderr,"Exit D4() ");
+   fprintf(stderr,"ExitD4() ");
    for(i=0;i<(len>40)?40:len;i++)
       if ( isprint(shmem->buf[i]) )
          fprintf(stderr,"%c",shmem->buf[i]);
@@ -1736,10 +1762,16 @@ static int resetPrinterOld(int deviceHdl, int prot)
 {
    int len;
    int  resetCommandLen = 26;
+#if 1
    if  (  (prot & PROT_EXIT )  )
+#else
+   if  (  initializeD4  )
+#endif
       exitD4(deviceHdl);
 
+#if 1
    if ( !(prot & PROT_D4) )
+#endif
    {
       SET_TIMER(ti,oti,500);
       len = write(deviceHdl, resetCommand, resetCommandLen);
@@ -1749,6 +1781,7 @@ static int resetPrinterOld(int deviceHdl, int prot)
          return 1;
       }
    }
+#if 1
    else
    {
        if ( ! d4Initialized )
@@ -1760,7 +1793,7 @@ static int resetPrinterOld(int deviceHdl, int prot)
          return 1;
       }
    }
-
+#endif
    sleep(3);
 
    return 0;
