@@ -176,7 +176,6 @@ static errorMessage_t errorMessage[] =
                                 sig = signal(SIGALRM, sigAlarm); \
                               }
 
-/*#if DEBUG*/
 /*******************************************************************/
 /* Function printHexValues                                         */
 /*                                                                 */
@@ -199,8 +198,6 @@ void printHexValues(char *dir, uc *buf, int len)
       fprintf(stderr,"%c  ",isprint(buf[i])&&!isspace(buf[i])?buf[i]:' ');
    fprintf(stderr,"\n");
 }
-/*#endif*/
-
 
 /*******************************************************************/
 /* Function sigAlarm(int code)                                     */
@@ -246,7 +243,7 @@ static int printError(uc errorNb)
    return 1; /* non recoverable error */
 }
 
-/*#if DEBUG*/
+
 /*******************************************************************/
 /* Function printCmdType()                                         */
 /*        print on stderr the command name                         */
@@ -280,7 +277,7 @@ static void printCmdType(uc *cmd)
       fprintf(stderr,"--- Send Data      ---\n");
    }
 }
-/*#endif*/
+
 
 /*******************************************************************/
 /* Function writeCmd()                                             */
@@ -298,7 +295,7 @@ static int writeCmd(int fd, char *cmd, int len)
    int w;
    int i = 0;
    struct  itimerval ti, oti;
-/*#if DEBUG*/
+
 # if PTIME
    struct timeval beg, end;
    long dt;
@@ -318,7 +315,7 @@ static int writeCmd(int fd, char *cmd, int len)
       printHexValues("Send: ",cmd,6);
       }
    }
-/*#endif*/
+
    usleep(1); /* according to Glen Steward, this will solve problems  */
               /* for the cartridge exchange with the Stylus Color 580 */
 
@@ -331,21 +328,18 @@ static int writeCmd(int fd, char *cmd, int len)
       RESET_TIMER(ti,oti);
       if ( w < 0 )
       {
-/*#if DEBUG*/
          if ( debugD4 )
          {
             fprintf(stderr,"Write error errno=%d ", errno);
             perror("");
          }
-/*#endif*/
          i= -1;
-         errno = ENODEV;
          break;
       }
       else
          i += w;
    }
-/*#if DEBUG*/
+
    if ( debugD4 )
    {
 # if PTIME
@@ -355,7 +349,7 @@ static int writeCmd(int fd, char *cmd, int len)
       fprintf(stderr,"Write time %5.3f s\n",(double)dt/1000000);
 # endif
    }
-/*#endif*/
+
    if ( timeoutGot )
       return -1;
    return i;
@@ -379,7 +373,7 @@ static int readAnswer(int fd, unsigned char *buf, int len)
    struct timeval beg, end;
    struct itimerval ti, oti;
    long dt;
-
+   int count = 0;
    /* wait a little bit before reading an answer */
    usleep(20000);
 
@@ -402,14 +396,22 @@ static int readAnswer(int fd, unsigned char *buf, int len)
          gettimeofday(&end, NULL);
          dt  = (end.tv_sec  - beg.tv_sec) * 1000;
          dt += (end.tv_usec - beg.tv_usec) / 1000;
-         if ( dt > d4RdTimeout )
+         if ( dt > d4RdTimeout || errno )
          {
-/*#if DEBUG*/
             if ( debugD4 )
-               fprintf(stderr,"Timeout at readAnswer()\n");
-/*# endif*/
+               fprintf(stderr,"Timeout 1 at readAnswer() rcv %d bytes\n",total);
+	    timeoutGot = 1;
             break;
          }
+	 count++;
+	 if ( count >= 20 )
+	 {
+	     timeoutGot = 1;
+	     if ( rd == 0 )
+	        errno = -1; /* tell that there is an abnormal coditon */
+	     break;
+	 }
+         errno = 0;
          continue;
       }
       else
@@ -425,14 +427,12 @@ static int readAnswer(int fd, unsigned char *buf, int len)
          }
       }
    }
-/*# if DEBUG*/
+   
    if ( debugD4 )
    {
 #  if PTIME
       gettimeofday(&end, NULL);
 #  endif
-/*# endif*/
-/*# if DEBUG*/
       printHexValues("Recv: ",buf,total);
 #  if PTIME
       dt = (end.tv_sec  - beg.tv_sec) * 1000000;
@@ -440,13 +440,10 @@ static int readAnswer(int fd, unsigned char *buf, int len)
       fprintf(stderr,"Read time %5.3f s\n",(double)dt/1000000);
 #  endif
    }
-/*# endif*/
    if ( timeoutGot )
    {
-/*#if DEBUG*/
       if ( debugD4 )
-         fprintf(stderr,"Timeout at readAnswer()\n");
-/*# endif*/
+         fprintf(stderr,"Timeout 2 at readAnswer()\n");
       return -1;
    }
    return total;
@@ -492,10 +489,9 @@ static int _readData(int fd, uc socketID, uc *buf, int len)
          dt += (end.tv_usec - beg.tv_usec) / 1000;
          if ( dt > d4RdTimeout*3 )
          {
-/*#if DEBUG*/
             if ( debugD4 )
                fprintf(stderr,"Timeout at _readData(), dt = %d ms\n",dt);
-/*#endif*/
+
             return -1;
             break;
          }
@@ -506,10 +502,9 @@ static int _readData(int fd, uc socketID, uc *buf, int len)
          total += rd;
       }
    }
-/*#if DEBUG*/
+
    if ( debugD4 )
       printHexValues("Recv: ",header,total);
-/*#endif*/
 
    if ( total == 6 )
    {
@@ -528,10 +523,8 @@ static int _readData(int fd, uc socketID, uc *buf, int len)
             dt += (end.tv_usec - beg.tv_usec) / 1000;
             if ( dt > d4RdTimeout*3 )
             {
-/*#if DEBUG*/
                if ( debugD4 )
                   fprintf(stderr,"Timeout at _readData(), dt = %d ms\n",dt);
-/*#endif*/
                return -1;
                break;
             }
@@ -542,10 +535,8 @@ static int _readData(int fd, uc socketID, uc *buf, int len)
             total += rd;
          }
       }
-/*#if DEBUG*/
       if ( debugD4 )
          printHexValues("Recv: ",buf,total);
-/*#endif*/
       return total;
    }
    return -1;
@@ -579,10 +570,8 @@ static int sendReceiveCmd(int fd, uc *cmd, int len, uc *answer, int expectedlen)
    else if ( rd < 0 )
    {
       /* interrupted write call */
-/*#if DEBUG*/
       if ( debugD4 )
          fprintf(stderr,"interrupt received\n");
-/*#endif*/
       return -1;
    }
    else
@@ -889,7 +878,7 @@ int CreditRequest(int fd, uc socketID)
 /* Function Credit()                                               */
 /*        give credit to the attached device                       */
 /* Input:  int   fd    file handle                                 */
-/*         uc    socketID  he socket to close                      */
+/*         uc    socketID  the socket to close                     */
 /*                                                                 */
 /* Return: -1 on error or 1 if all is OK                           */
 /*                                                                 */
@@ -940,19 +929,20 @@ int Credit(int fd, uc socketID, int credit)
 /*         the returned credit is 0                                */
 /*                                                                 */
 /*******************************************************************/
-
+#define MAX_CREDIT_REQUEST 2
 int askForCredit(int fd, uc socketID, int *sndSize, int *rcvSize)
 {
    int credit = 0;
-
+   int count  = 0;
+   
    while (credit == 0 )
    {
-      while((credit=CreditRequest(fd,socketID)) == 0 )
+      while((credit=CreditRequest(fd,socketID)) == 0  && count < MAX_CREDIT_REQUEST )
          usleep(250000);
 
       if ( credit == -1 )
       {
-         if ( errno == ENODEV )
+         if ( errno == ENODEV || count == MAX_CREDIT_REQUEST )
          {
             break;
          }
@@ -967,6 +957,7 @@ int askForCredit(int fd, uc socketID, int *sndSize, int *rcvSize)
       /* if the parent died, live this loop if credit not got */ 
       if ( credit == 0 && getppid() == ppid )
          return 0;
+      count++;
    }
    return credit;
 }
@@ -991,19 +982,15 @@ int writeData(int fd, uc socketID, uc *buf, int len, int eoj)
    int wr = 0;
    int ret = 0;
    struct  itimerval ti, oti;
-/*#if DEBUG*/
    struct timeval beg, end;
    long dt;
-/*#endif*/
    static uc *buffer = NULL;
    static int bLen   = 0;
-/*#if DEBUG*/
    if ( debugD4 )
    {
       fprintf(stderr,"--- Send Data      ---\n");
       gettimeofday(&beg, NULL);
    }
-/*#endif*/
    len += 6;
    if ( len > bLen )
    {
@@ -1032,7 +1019,6 @@ int writeData(int fd, uc socketID, uc *buf, int len, int eoj)
       if ( ret == -1 )
       {
          perror("write: ");
-         errno = ENODEV;
       }
       else
       {
@@ -1040,7 +1026,6 @@ int writeData(int fd, uc socketID, uc *buf, int len, int eoj)
       }
    }
 
-/*#if DEBUG*/
    if ( debugD4 )
    {
 # if PTIME
@@ -1059,7 +1044,7 @@ int writeData(int fd, uc socketID, uc *buf, int len, int eoj)
        fprintf(stderr,"Write time %5.3f s\n",(double)dt/1000000);
 # endif
    }
-/*#endif*/
+
    if (  wr > 6 )
       wr -= 6;
    else
