@@ -697,6 +697,8 @@ void exchange_CB(Widget w, XtPointer clientData, XtPointer callData)
 void reset_CB(Widget w, XtPointer clientData, XtPointer callData)
 {
    callPrg(RESET_PRT, actConfig.dev, actConfig.reset, 0, 0, NULL, NULL);
+   /* avoid that anythings called before the printer was really reseted */
+   cmdCount = - 10;
 }
 
 /*******************************************************************/
@@ -2556,6 +2558,9 @@ static void help_CB(Widget w, XtPointer clientData, XtPointer callData)
    char **dir;
    char  *dirs [] = {
        localDir,
+#if defined PREFIX
+       PREFIX"/doc/mtink",
+#endif
        "/usr/share/doc/printer-utils-1.0/EpsonInkjetMaintenance",
        "/usr/local/share/doc/mtink",
        "/usr/local/share/doc",
@@ -2575,7 +2580,7 @@ static void help_CB(Widget w, XtPointer clientData, XtPointer callData)
     getcwd(localDir, sizeof(localDir));
     home[0] = '\0';
     if ( myCastel )
-       sprintf(home,"%s/mtink",myCastel);
+       snprintf(home,sizeof(localDir)-100,"%s/mtink",myCastel);
     dir = dirs;
     /* look for the mtink.html file */
     while( dir[0] )
