@@ -342,6 +342,7 @@ char *_searchString(char *lang, char *key)
    char *defaultMsg   = NULL;
    char *localizedMsg = NULL;
    char  *s;
+   int utf;
 
    while( *msg )
    {
@@ -351,7 +352,11 @@ char *_searchString(char *lang, char *key)
       if ( *s ) s++;
       if ( *s && lang )
       {
-         if ( s[0] == lang[0] &&  s[1] == lang[1] &&
+         if ( strstr(lang,"8" ) )
+            utf = 1;
+         else
+            utf = 0;
+         if ( !utf && s[0] == lang[0] &&  s[1] == lang[1] &&
               (s[2] == '.' || s[2] == '*')
             )
          {
@@ -362,6 +367,18 @@ char *_searchString(char *lang, char *key)
                return localizedMsg;
             }
             
+         }
+         else if ( utf && s[0] == lang[0] &&  s[1] == lang[1] && s[2] == lang[2]  &&
+              (s[3] == '.' || s[3] == '*')
+            )
+         {
+            /* may be that we have found the wanted string */
+            
+            if ( (localizedMsg = checkRessourceString(s+4, key)) )
+            {
+               return localizedMsg;
+            }
+            
          }
          else
          {
@@ -455,6 +472,7 @@ void printText(char *fmt, char *lang, char *key)
 void fprintText(FILE *out, char *fmt, char *lang, char *key)
 {
    char *msg;
+
    msg = searchString(lang, key);
    if ( msg )
       fprintf(out,fmt, msg);
