@@ -73,6 +73,7 @@ int doAlign             = 0;
 int doExchangeCartridge = 0;
 int cycle               = 0;
 int doIdent             = 0;
+int utf8                = 0;
 
 int d4Printer           = 0;
 char *model             = NULL;
@@ -89,6 +90,7 @@ int exchangeInk(char *lang, configData_t actConfig);
 
 int setConfig(char *lang);
 
+int (*ynFunc)(char*,char*);
 
 
 /*******************************************************************/
@@ -135,10 +137,10 @@ void setPrinterStateLabel(int state)
 /*                                                                 */
 /*******************************************************************/
 
-void exit_CB(void)
+void exit_CB(int val)
 {
    callPrg(TERMINATE,NULL, 0, 0, 0, NULL, NULL);
-   exit(0);
+   exit(val);
 }
 
 /*******************************************************************/
@@ -184,6 +186,10 @@ void parseArgs(int argc, char **argv, char *lang)
       {
          printSyntax(0, lang);
       }
+      if ( strncmp("-W", argv[0], 2) == 0 )
+      {
+         ynFunc = askYn;
+      }
       else if ( strncmp("-d", argv[0], 2) == 0 ||
                 strncmp("--d", argv[0], 3) == 0 )
       {
@@ -286,6 +292,13 @@ void parseArgs(int argc, char **argv, char *lang)
           doStatus = 1;
 
       }
+      else if ( strncmp("-u", argv[0], 2) == 0 ||
+                strncmp("--u", argv[0], 3) == 0 )
+      {
+          /* printer status command*/
+          utf8 = 1;
+          lang[2] = '8'; lang[3] = '\0';
+      }
       else if ( strncmp("-L", argv[0], 2) == 0  )
       {
           /* printer status command*/
@@ -355,6 +368,9 @@ int main(int argc, char **argv)
    char           *retBuf       = NULL;
    char            printerFound = 0;
    char           *lang         = NULL;
+   int             retVal       = 0;
+
+   ynFunc = askYn;
 
    prgName = argv[0];
    if ( (s = strrchr(prgName, '/')) != NULL )
@@ -390,7 +406,12 @@ int main(int argc, char **argv)
       s = "en";
    }
 
-   lang = (char *)malloc(3);
+   if ( strstr(lang, "UTF-8") )
+   {
+      utf8 = 1;
+   }
+
+   lang = (char *)malloc(4);
    if ( lang == NULL )
    {
       exit(1);
@@ -400,10 +421,21 @@ int main(int argc, char **argv)
    lang[0] = toupper(s[0]);
    lang[1] = s[1];
    lang[2] = '\0';
-   initResource();
+   if ( utf8 )
+   {
+      lang[2] = '8';
+      lang[3] = '\0';
+   }
 
+   initResource();
+   
    readPrinterData();
    parseArgs(argc,argv, lang);
+   if ( utf8 )
+   {
+      lang[2] = '8';
+      lang[3] = '\0';
+   }
 
    signal(SIGINT,  sigKill);
    signal(SIGQUIT, sigKill);
@@ -417,7 +449,7 @@ int main(int argc, char **argv)
       if ( state == -1 )
       {
           fprintText(stderr,"%s\n", lang, "noAccess");
-          exit_CB();         
+          exit_CB(1);         
       }
       else if ( state == 0 && retBuf )
       {
@@ -513,7 +545,7 @@ int main(int argc, char **argv)
    if ( printerFound == 0 && (!d4Printer && doIdent) )
    {
       printText("%s\n",lang, "noDetected");
-      exit_CB();
+      exit_CB(1);
    }
    if ( printerFound == 0 )
    {
@@ -548,11 +580,11 @@ int main(int argc, char **argv)
    if ( doStatus )
    {
 loop:
-      printf("%s:\n", actConfig.name);
       retBuf = NULL;
       callPrg(GET_IQ, actConfig.dev, actConfig.prot, 0, 0, &retBuf, NULL);
-      if ( retBuf != NULL )
+      if ( retBuf != NULL && *retBuf )
       {
+         printf("%s:\n", actConfig.name);
          if ( *retBuf == '\0' )
          {
             printerFound = 0;
@@ -576,7 +608,7 @@ loop:
       }
       else
       {
-         if ( !actConfig.id )
+         //if ( !actConfig.id || )
          {
             printerFound = 0;
          }
@@ -585,25 +617,29 @@ loop:
       if ( printerFound == 0 )
       {
          printText("%s\n", lang, "noOPen");
+         exit_CB(1);
       }
-      
    }
 
    if ( doCheck )
    {
-      callPrg(CHECK_NOZZLE, actConfig.dev,actConfig.prot, 0, 0, NULL, NULL);
+      retVal = callPrg(CHECK_NOZZLE, actConfig.dev,actConfig.prot, 0, 0, NULL, NULL);
    }
 
    if ( doClean )
    {
-      callPrg(CLEAN_NOZZLE, actConfig.dev,actConfig.prot, 0, 0, NULL, NULL);
+      retVal = callPrg(CLEAN_NOZZLE, actConfig.dev,actConfig.prot, 0, 0, NULL, NULL);
    }
 
    if ( doReset )
    {
-      callPrg(RESET_PRT, actConfig.dev, actConfig.reset, 0, 0, NULL, NULL);
+      retVal = callPrg(RESET_PRT, actConfig.dev, actConfig.reset, 0, 0, NULL, NULL);
+   }
+   if ( retVal < 0 )
+   {
+      retVal = 1;
+      printText("%s\n", lang, "noOPen");
    }
-
    /* from here inter actice functions */
 
    if ( doAlign )
@@ -616,7 +652,7 @@ loop:
       exchangeInk(lang, actConfig);
    }
 
-   exit_CB();
+   exit_CB(0);
 
    return 0;
 }
