@@ -5,12 +5,12 @@
 # Set SEARCH to the line which will precede the new entry
 # \ must be duplicated
 
-SEARCH='Steven J. Mackenzie\\n\\'
+SEARCH='Ronny Budzinske\\n\\'
 
 # Set APPEND to the line which will is to be added
 # \ must be duplicated
 
-APPEND='Raul Morales\\n\\   raul.mh@telefonica.net\\n\\   http://www.telefonica.net/web/ruten\\n\\'
+APPEND='Nicola Fabiano\\n\\   ivololeribar@yahoo.it\\n\\'
 for f in Mtink.??
 do
    awk '{
